function vlinregr(dataFile, nData, netName, aviFile,nIter)
% *************************************************************************
% Linear Regression Demo in Vibes in 2 Dimensions
% -----------------------------------------------
% vlinregr(dataFile,nData, netName, aviFile,nIter)
%
% dataFile = source of data on which regression is to be performed.
% example data: http://www.ics.uci.edu/~mlearn/MLRepository.html
% nData = Number of data samples to take
% netName =  name of the vibes bayesian network in .xml format.
% aviFile = name of the movie file to create. 
% nIter = number of iterations of variational inference algorithm.
%
% Regression equation followed: f(x,w)= w0 + sum_i(w*x)
% The problem is to estimate vector w so as to fit a target variable y
% where x is my training data
% *************************************************************************
% Author: Anirban Sinha, UBC Grad student, Email: anirbans@cs.ubc.ca
% Last modified: Nov 26 2005, 9:30 AM.
% Some code snippets are taken from Kevin Murphy's solution to HW 3 &
% matlab help files.
% Really grateful to Kevin Murphy & Frank Hutter for their continous help & 
% suggestions.
% Also grateful to John Winn (developer of Vibes) for helping me with 
% questions regarding Vibes.
% *************************************************************************
fprintf('Starting ...');
% Load data (example data: http://www.ics.uci.edu/~mlearn/MLRepository.html)


data=load(dataFile);
% ************
% Let d be the number of dimensions to take from training data & N be the 
% number of training
% data
% ************
[N nCols]=size(data);
if nData<N
    N=nData;
end
d=1;

% Formatting training data & target variable

x=data(1:N,[1:d]); % training variable
y=data(1:N,end); % target variable
[x, mu, sigma] = standardizeCols(x);
x = [ones(N,1) x];
d=d+1; % imediately dimension of x increases by 1 
save('data.mat'); % save environment before more variables are initialized

% Creating grid to plot the posterior Gaussian
stepSize = 0.5;
[i,j] = meshgrid(-5:stepSize:25,-5:stepSize:25); % Create grid.
[r,c]=size(i);
datamesh = [i(:) j(:)];

      
% **** Actual Initialization & Inference iterations begin from here ...

fprintf('\nInitializing Vibes & loading network & data ...');

javaaddpath 'Vibes2_0_1.jar';
import cam.jmw39.app.vibes.matlab.*;
net=Vibes.read(netName);

net.getPlate('N').setSize(N);
net.getPlate('d').setSize(d);
load('data.mat');
net.getNode('x').setData(x);
net.getNode('y').setData(y);

algo=Vibes.init(net);

mu=repmat(0,d,1); % initialize mean of gaussian as all zeros

% Creating movie file for demo 
mov = avifile(aviFile);
fig=figure;
set(fig,'DoubleBuffer','on');
set(gca,'xlim',[-80 80],'ylim',[-80 80],'NextPlot','replace','Visible','off');
fprintf('\nIterating in Var Inf. ...');
% plot the intial posterior of w 

w=Vibes.get(net,'w');
cov=diag(w(:,2) - w(:,1).^2); % cov=diag(var) becoz w's are uncorrelated
mu=w(:,1);
p = gausspdf(datamesh, mu, cov);
p = reshape(p, r, c);
p=p* stepSize^2; % p2(x,y) defeq p(x: x+dx, y: y+ dy) approx p(x,y) dx dy
h=surfc(i,j,p);
set(h,'EraseMode','xor'); % set graphics handle to XOR mode
F = getframe(gca); % Get current frame
mov = addframe(mov,F); % add frame to movie

% single stepping - wait for user to press any key ...
fprintf('\nPress any key to proceed ...');
pause

% now iterate ...
for iter=1:nIter
    algo.update(3);
    w=Vibes.get(net,'w')
    cov=diag(w(:,2) - w(:,1).^2); % cov=diag(var) becoz w's are uncorrelated.
    mu=w(:,1);
    
    p = gausspdf(datamesh, mu, cov);
    p = reshape(p, r, c);
    p=p* stepSize^2; % p2(x,y) defeq p(x: x+dx, y: y+ dy) approx p(x,y) dx dy
    
    h=surfc(i,j,p);
    set(h,'EraseMode','xor'); % set graphics handle to XOR mode
    F = getframe(gca); % Get current frame
    mov = addframe(mov,F); % add frame to movie
    % single stepping - wait for user to press any key ...
    fprintf('\nPress any key to proceed ...');
    pause
end

% Printing results ...
fprintf('\nEstimated value of w through inference algorithm\n');
w=Vibes.get(net,'w');
fprintf('%3.6f \n',w(:,1));

% & doing some cleanup job 
mov = close(mov);

% *** Done :) ***


winf=w(:,1);
% ** Anaytical Exact estimation of w
w = pinv(x'*x)* x' * y;
fprintf('\nExact estimation of linear weight vector is:\n');
fprintf('%3.6f \n',w);

assert(approxeq(winf,w));
