function vlinregr(dataFile, nData, netName, nIter)
% *************************************************************************
% Linear Regression Demo in Vibes in 1 Dimension
% Since Vibes assumes univariate posterior so, plotting
% posterior dimensions as seperate univariate gaussians.
% This time removing movie stuff as was suggested by Kevin.
% -------------------------------------------------------------------------
% vlinregr(dataFile,nData, netName, aviFile,nIter)
%
% dataFile = source of data on which regression is to be performed.
% example data: http://www.ics.uci.edu/~mlearn/MLRepository.html
% nData = Number of data samples to take
% netName =  name of the vibes bayesian network in .xml format. 
% nIter = number of iterations of variational inference algorithm.
%
% Regression equation followed: f(x,w)= w0 + sum_i(w*x)
% The problem is to estimate vector w so as to fit a target variable y
% where x is my training data
% *************************************************************************
% Author: Anirban Sinha, UBC Grad student, Email: anirbans@cs.ubc.ca
% Last modified: Dec 10 2005, 6:30 PM.
% Some code snippets are taken from Kevin Murphy's solution to HW 3 &
% matlab help files.
% Really grateful to Kevin Murphy & Frank Hutter for their continous help & 
% suggestions.
% Also grateful to John Winn (developer of Vibes) for helping me with 
% questions regarding Vibes.
% *************************************************************************
fprintf('Starting ...');

% Load data (example data: http://www.ics.uci.edu/~mlearn/MLRepository.html)

data=load(dataFile);
% ************
% Let d be the number of dimensions to take from training data & N be the 
% number of training
% data
% ************
[N nCols]=size(data);
if nData<N
    N=nData;
end
d=1;

% Formatting training data & target variable

x=data(1:N,[1:d]); % training variable
y=data(1:N,end); % target variable
[x, mu, sigma] = standardizeCols(x);
x = [ones(N,1) x];
d=d+1; % imediately dimension of x increases by 1 
save('data.mat'); % save environment before more variables are initialized

% Creating grid to plot the posterior Gaussian
stepSize = 1;

datamesh = [-25:stepSize:25];
datamesh=datamesh';
[r,c]=size(datamesh);
      
% **** Actual Initialization & Inference iterations begin from here ...

fprintf('\nInitializing Vibes & loading network & data ...');

javaaddpath 'Vibes2_0_1.jar';
import cam.jmw39.app.vibes.matlab.*;
net=Vibes.read(netName);

net.getPlate('N').setSize(N);
net.getPlate('d').setSize(d);
load('data.mat');
net.getNode('x').setData(x);
net.getNode('y').setData(y);

algo=Vibes.init(net);

mu=repmat(0,d,1); % initialize mean of gaussian as all zeros


fprintf('\nIterating in Var Inf. ...');
% plot the intial posterior of w 

w=Vibes.get(net,'w');
cov=diag(w(:,2) - w(:,1).^2); % cov=diag(var) becoz w's are uncorrelated
mu=w(:,1);

% Plot first dimension ...
p = gausspdf(datamesh, mu(1), cov(1,1));
p = reshape(p, r, c);
p=p * stepSize;  %normalize p
subplot(1,2,1);
plot(datamesh,p);
title('Posterior distribution of W[0]');
xlabel('x','fontsize',15);
ylabel('N(x)','fontsize',15);

% Plot second Dimension ...

p = gausspdf(datamesh, mu(2), cov(2,2));
p = reshape(p, r, c);
p=p*stepSize; % normalize p
subplot(1,2,2);
plot(datamesh,p);
title('Posterior distribution of W[1]');
xlabel('x','fontsize',15);
ylabel('N(x)','fontsize',15);

% single stepping - wait for user to press any key ...
fprintf('\nPress any key to proceed ...');
pause

% now iterate ...
for iter=1:nIter
    algo.update(3);
    w=Vibes.get(net,'w')
    cov=diag(w(:,2) - w(:,1).^2) % cov=diag(var) becoz w's are uncorrelated.
    mu=w(:,1)
    
    % Plot first dimension ...
    
    p = gausspdf(datamesh, mu(1), cov(1,1));
    p = reshape(p, r, c);
    p=p/ max(p); % normalize p
    subplot(1,2,1);
    plot(datamesh,p);
    title('Posterior distribution of W[0]');
    xlabel('x','fontsize',15);
    ylabel('N(x)','fontsize',15);
    
    % Plot second Dimension ...
    p = gausspdf(datamesh, mu(2), cov(2,2));
    p = reshape(p, r, c);
    p=p/ max(p); % normalize p
    subplot(1,2,2);
    plot(datamesh,p);
    title('Posterior distribution of W[1]');
    xlabel('x','fontsize',15);
    ylabel('N(x)','fontsize',15);
    
    % single stepping - wait for user to press any key ...
    fprintf('\nPress any key to proceed ...');
    pause
end

% Printing results ...
fprintf('\nEstimated value of w through inference algorithm\n');
w=Vibes.get(net,'w');
fprintf('%3.6f \n',w(:,1));


% *** Done :) ***


winf=w(:,1);
% ** Anaytical Exact estimation of w
w = pinv(x'*x)* x' * y;
fprintf('\nExact estimation of linear weight vector is:\n');
fprintf('%3.6f \n',w);

assert(approxeq(winf,w));
